/**
 * @file
 * Enables spellchecking via Yandex spellcheck service.
 */

(function ($, Drupal) {
  'use strict';

  Drupal.behaviors.ckeditor_yandex_spellchecker = {
    attach: function (context, settings)
    {
      CKEDITOR.on('instanceReady', function(ev)
      {
        var editor = ev.editor;

        if ( !editor.spellCheckInited )
        {
          var editorBottomSelector = '#' + editor.id + '_bottom';
          var editorBottom = $(editorBottomSelector);
          var spellCheckBar = $('<div>').addClass('spell-check-bar');
          editorBottom.prepend(spellCheckBar);
          editor.spellCheckBar = spellCheckBar;
          editor.spellCheckInited = true;
        }

        function clearSuggestions()
        {
          var suggestions = editor.spellCheckBar.find('.suggestion');
          if ( suggestions.length ){
            suggestions.each(function(num, el) {
              $(el).remove();
            });
          }
        }

        function newSuggestion(wrong, fix, text)
        {
          var newSugg = $('<span>');
          newSugg.addClass('suggestion');
          newSugg.text(wrong + ' ~> ' + fix);
          $(newSugg).on('click', function (ev) {
            text.data = text.data.replace(wrong, fix);
            $(ev.target).remove();
          });
          return newSugg;
        }

        function updateSuggestions(data, focusNode)
        {
          clearSuggestions();
          if ( data.length ){
            data.forEach(function( item ){
              if ( item.s.length ){
                var newSugg = newSuggestion(item.word, item.s[0], focusNode);
                $(editor.spellCheckBar).append(newSugg);
              }
            });
          }
        }

        function spellCheck()
        {
          clearSuggestions();

          var sel = editor.getSelection();
          if ( !sel ){ return; }

          var nativeSel = sel._.cache.nativeSel || null;
          if ( !nativeSel ){ return; }

          var focusNode = nativeSel.focusNode;
          if ( !focusNode ){ return; }

          var text = focusNode.data;
          if ( !text ){ return; }

          $.get('http://speller.yandex.net/services/spellservice.json/checkText?text=' + text,function (data) {
            updateSuggestions(data, focusNode)
          });
        }

        editor.document.on('mouseup', function (evt) { spellCheck(); });
        editor.on('change', function (evt) { spellCheck(); });

      });
    }
  };

})(jQuery, Drupal);
